"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@coral-xyz/anchor");
const web3_js_1 = require("@solana/web3.js");
const fs_1 = __importDefault(require("fs"));
const mangoAccount_1 = require("../../src/accounts/mangoAccount");
const builder_1 = require("../../src/builder");
const client_1 = require("../../src/client");
const clientIxParamBuilder_1 = require("../../src/clientIxParamBuilder");
const constants_1 = require("../../src/constants");
//
// This script creates liquidation candidates
//
const GROUP_NUM = Number(process.env.GROUP_NUM || 200);
const CLUSTER = process.env.CLUSTER || 'mainnet-beta';
// native prices
const PRICES = {
    ETH: 1200.0,
    SOL: 0.015,
    USDC: 1,
    MNGO: 0.02,
};
const TOKEN_SCENARIOS = [
    [
        'TCS, FUNDING',
        [
            ['USDC', 5000000],
            ['ETH', 100000],
            ['SOL', 150000000],
        ],
        [],
    ],
    ['TCS, LIQOR', [['USDC', 100000000]], []],
];
const NUM_ACCOUNTS = 4;
const NUM_TCS_PER_ACCOUNT = 20;
async function main() {
    const options = anchor_1.AnchorProvider.defaultOptions();
    options.commitment = 'processed';
    options.preflightCommitment = 'finalized';
    const connection = new web3_js_1.Connection(process.env.CLUSTER_URL, options);
    const admin = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(process.env.PAYER_KEYPAIR, 'utf-8'))));
    const userWallet = new anchor_1.Wallet(admin);
    const userProvider = new anchor_1.AnchorProvider(connection, userWallet, options);
    const client = await client_1.MangoClient.connect(userProvider, CLUSTER, constants_1.MANGO_V4_ID[CLUSTER], {
        idsSource: 'get-program-accounts',
        prioritizationFee: 100,
        txConfirmationCommitment: 'confirmed',
    });
    console.log(`User ${userWallet.publicKey.toBase58()}`);
    // fetch group
    const group = await client.getGroupForCreator(admin.publicKey, GROUP_NUM);
    console.log(group.toString());
    const MINTS = new Map([
        ['USDC', group.banksMapByName.get('USDC')[0].mint],
        ['ETH', group.banksMapByName.get('ETH')[0].mint],
        ['SOL', group.banksMapByName.get('SOL')[0].mint],
    ]);
    const accounts = await client.getMangoAccountsForOwner(group, admin.publicKey);
    let maxAccountNum = Math.max(0, ...accounts.map((a) => a.accountNum));
    async function createMangoAccount(name) {
        const accountNum = maxAccountNum + 1;
        maxAccountNum = maxAccountNum + 1;
        await client.createMangoAccount(group, accountNum, name, 4, 4, 4, 4);
        const account = (await client.getMangoAccountForOwner(group, admin.publicKey, accountNum));
        await client.accountExpandV2(group, account, 4, 4, 4, 4, 50);
        await account.reload(client);
        return account;
    }
    async function setBankPrice(bank, price) {
        await client.stubOracleSet(group, bank.oracle, price);
        // reset stable price
        await client.tokenEdit(group, bank.mint, (0, builder_1.Builder)(clientIxParamBuilder_1.NullTokenEditParams).resetStablePrice(true).build());
    }
    async function setPerpPrice(perpMarket, price) {
        await client.stubOracleSet(group, perpMarket.oracle, price);
        // reset stable price
        await client.perpEditMarket(group, perpMarket.perpMarketIndex, (0, builder_1.Builder)(clientIxParamBuilder_1.NullPerpEditParams).resetStablePrice(true).build());
    }
    for (const scenario of TOKEN_SCENARIOS) {
        const [name, assets, liabs] = scenario;
        // create account
        console.log(`Creating mangoaccount...`);
        const mangoAccount = await createMangoAccount(name);
        console.log(`...created mangoAccount ${mangoAccount.publicKey} for ${name}`);
        for (const [assetName, assetAmount] of assets) {
            const assetMint = new web3_js_1.PublicKey(MINTS.get(assetName));
            await client.tokenDepositNative(group, mangoAccount, assetMint, new anchor_1.BN(assetAmount));
            await mangoAccount.reload(client);
        }
        for (const [liabName, liabAmount] of liabs) {
            const liabMint = new web3_js_1.PublicKey(MINTS.get(liabName));
            // temporarily drop the borrowed token value, so the borrow goes through
            const bank = group.banksMapByName.get(liabName)[0];
            try {
                await setBankPrice(bank, PRICES[liabName] / 2);
                await client.tokenWithdrawNative(group, mangoAccount, liabMint, new anchor_1.BN(liabAmount), true);
            }
            finally {
                // restore the oracle
                await setBankPrice(bank, PRICES[liabName]);
            }
        }
    }
    for (let i = 0; i < NUM_ACCOUNTS; i++) {
        const name = `TCS, A${i}`;
        console.log(`Creating mangoaccount...`);
        const mangoAccount = await createMangoAccount(name);
        console.log(`...created mangoAccount ${mangoAccount.publicKey} for ${name}`);
        const assetMint = new web3_js_1.PublicKey(MINTS.get('USDC'));
        await client.tokenDepositNative(group, mangoAccount, assetMint, new anchor_1.BN(1000000));
        await mangoAccount.reload(client);
        for (let i = 0; i < NUM_TCS_PER_ACCOUNT; i++) {
            await client.tokenConditionalSwapCreateRaw(group, mangoAccount, MINTS.get('SOL'), MINTS.get('USDC'), new anchor_1.BN(100000000), new anchor_1.BN(20000), null, 0.0, 1000000.0, 0.01, true, true, mangoAccount_1.TokenConditionalSwapDisplayPriceStyle.buyTokenPerSellToken, mangoAccount_1.TokenConditionalSwapIntention.unknown);
        }
    }
    process.exit();
}
main();
